#version 420 core

uniform mat4 model;
uniform vec3 cameraPosition;

layout(location = 0) out vec4 finalColor;

layout(binding = 0) uniform sampler2D material_texture0;
layout(binding = 1) uniform sampler2D color_texture1;
layout(binding = 2) uniform sampler2DShadow shadowMap;

uniform float materialShininess;
uniform vec3 materialSpecularColor;

in vec2 fragTexCoord;
in vec3 fragNormal;
in vec3 fragVert;
in vec4 ShadowCoord;

uniform struct Light
{
	vec3 position;
	vec3 intensities; //the color of the light
	float attenuation;
	float ambientCoefficient;
} light;

void main()
{
	//light.attenuation = attenuation;
	//light.ambientCoefficient = ambientCoefficient;

	// Shadow stuff
	float visibility = texture( shadowMap, vec3(ShadowCoord.xy, (ShadowCoord.z)/ShadowCoord.w) );

	int isThisFragVisible = 0;
	if(texture(color_texture1, fragTexCoord).a != 0.0f)
		isThisFragVisible = 1;

	float mask = 1.0f * texture(color_texture1, fragTexCoord).a;
	vec4 texture = texture(material_texture0, fragTexCoord);

	float r = texture.r + mask;
	float g = texture.g + mask;
	float b = texture.b + mask;
	
	if(r > 1.0f) r = 1.0f;
	if(g > 1.0f) g = 1.0f;
	if(b > 1.0f) b = 1.0f;
	
	vec4 texColor = vec4(r, g, b, 1.0f);

	if(false){ //DEBUG
		finalColor = texture * visibility;	
	} else{
		if(isThisFragVisible == 1)
		{
			finalColor = texColor;
		}else
		{
			//
			//	Light Calculation (Phong model)
			//
			vec3 normal = normalize(transpose(inverse(mat3(model))) * fragNormal); //calculate normal in world coordinates
			vec3 surfacePosition = vec3(model * vec4(fragVert, 1)); //calculate the location of this fragment (pixel) in world coordinates
			vec4 surfaceColor = texColor;
			vec3 surfaceToLight = normalize(light.position - surfacePosition); //calculate the vector from this pixels surface to the light source
			vec3 surfaceToCamera = normalize(cameraPosition - surfacePosition);

			//calculate ambient light
			vec3 ambient = light.ambientCoefficient * surfaceColor.rgb * light.intensities; 

			//calculate diffuse light
			float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight)); //calculate the cosine of the angle of incidence
			vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.intensities;


			//calculate specular light
			float specularCoefficient = 0.0;
			float normalizeFactor = 1;

			if(diffuseCoefficient > 0.0)
			{ //only calc specularCoefficient for front of surface
				specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), materialShininess);
		
				float pi = 3.14159265359;
				normalizeFactor = (materialShininess +2) / (2*pi);
			}
			vec3 specular = light.intensities * specularCoefficient * normalizeFactor * materialSpecularColor;

			if(materialShininess == 0.0)
			{
				specular = vec3(0,0,0);
			}

			//attenuation - distance correction for diffuse and specular light
			float distanceToLight = length(light.position - surfacePosition);
			float attenuation = 1.0 / (1.0 + light.attenuation * pow(distanceToLight, 2));

			//linear color (color before gamma correction)
			vec3 linearColor = ambient + attenuation*(diffuse + specular);

			//final color (color after gamma correction)
			vec3 gamma = vec3(1.0/2.2);
			finalColor = vec4(pow(linearColor, gamma), surfaceColor.a) * visibility;
		}
	}
}